<?php

namespace Freshmail\Core\AdminMenu\Pages;

use Freshmail\Core\Api;

use function App\template;

class ApiSettingsPage extends AbstractPage
{
  /**
   * Returns page setup data - sections and fields arrays.
   * Can't define as const var because of callbacks.
   *
   * @return array
   * @author MS
   */
  protected function getSetup(): array
  {
    return [
      'subpages' => [
        [
          'parent_slug' => 'freshmail_settings',
          'page_title' => _x('Connect', 'Settins page title', 'freshmail'),
          'menu_title' => _x('Integration', 'Menu label', 'freshmail'),
          'capability' => 'manage_options',
          'menu_slug' => 'freshmail_settings',
          'callback' => [$this, 'renderPage'],
          'position' => null,
        ]
      ],
      'sections' => [
        [
          'id' => 'f_api_main_settings_section',
          'title' => '',//__('Connect', 'freshmail'),
          'callback' => [$this, 'renderSettingsSection'],
          'page' => 'freshmail-api-settings',
        ],
      ],
      'fields' => [
        [
          'label' => __('Beaer Token', 'freshmail'),
          'type' => 'password',
          'name' => 'f_bearer_token',
          'id' => 'f_bearer_token',
          'css_class' => 'api-setting-field',
          'value' => $this->getValue_f_bearer_token(''),
          'placeholder' => '',
          'callback' => [$this, 'renderSettingsField_f_bearer_token'],
          'page' => 'freshmail-api-settings',
          'section' => 'f_api_main_settings_section',
          'args' => [],
          'additionalHtml' => $this->getBearerTokenAdditionalHtml(),
        ],
        [
          'label' => __('Subscribers list', 'freshmail'),
          'type' => 'select',
          'name' => 'f_subscribers_list_hash',
          'id' => 'f_subscribers_list_hash',
          'css_class' => 'api-setting-field',
          'value' => $this->getValue_f_subscribers_list_hash(''),
          'options' => self::getOptionsForSubscribersList(),
          'placeholder' => '',
          'additionalHtml' => $this->getSubscribersListAdditionalHtml(),
          // 'description' => __('Subscribers list that will be used to save subscribers. (list is refreshed every hour)', 'freshmail'),
          'callback' => [$this, 'renderSettingsField_f_subscribers_list_hash'],
          'page' => 'freshmail-api-settings',
          'section' => 'f_api_main_settings_section',
          'args' => [],
        ],
        [
          'label' => __('New subscriber status', 'freshmail'),
          'type' => 'select',
          'name' => 'f_new_subscriber_status',
          'id' => 'f_new_subscriber_status',
          'css_class' => 'api-setting-field',
          'value' => $this->getValue_f_new_subscriber_status(),
          'options' => $this->getOptionsForNewSubscriberStatus(),
          'placeholder' => '',
          'description' => __('Whether new subscribers should be added to your list as already activated or awaiting activation.', 'freshmail'),
          'callback' => [$this, 'renderSettingsField_f_new_subscriber_status'],
          'page' => 'freshmail-api-settings',
          'section' => 'f_api_main_settings_section',
          'args' => [],
        ],
      ],
    ];
  }

  /**
   * Renders this page
   *
   * @return void
   * @author MS
   */
  public function renderPage()
  {
    include FRERSHMAIL_RESOURCES_PATH . 'views/admin/settings/api-settings.php';
  }

  /**
   * Returns additional HTML for subscribers list setting
   *
   * @return string
   * @author MS
   */
  private function getBearerTokenAdditionalHtml()
  {
    ob_start();
    include FRERSHMAIL_RESOURCES_PATH . 'views/admin/settings/api-settings/bearer-token-additional.php';
    return ob_get_clean();
  }

  /**
   * Returns additional HTML for subscribers list setting
   *
   * @return string
   * @author MS
   */
  private function getSubscribersListAdditionalHtml()
  {
    ob_start();
    include FRERSHMAIL_RESOURCES_PATH . 'views/admin/settings/api-settings/subscribers-list-additional.php';
    return ob_get_clean();
  }

  /**
   * Returns options for Subscribers list select
   *
   * @param bool $force_refresh_transient if true then transient will be refreshen with new data from API
   * @return array
   * @author MS
   */
  public static function getOptionsForSubscribersList($force_refresh_transient = false): array
  {
    $lists = get_transient('f_subscribers_lists');
    if ((empty($lists) || $force_refresh_transient) && ! empty(get_option('f_bearer_token', ''))) {
      $lists = [];
      $lists_raw = freshmail('api')->subscribers_list->getAvailableLists();
      if (empty($lists_raw)) {
        $lists_raw[] = freshmail('api')->subscribers_list->createList();
      }
      foreach ($lists_raw as $list) {
        $lists[] = [
          'label' => $list['name'],
          'value' => $list['subscriberListHash'],
        ];
      }
      set_transient('f_subscribers_lists', $lists, HOUR_IN_SECONDS);
    }
    return empty($lists) ? [] : $lists;
  }

  /**
   * Ajax: Refresh FM subscribers lists
   *
   * @action wp_ajax_f_refresh_subscribers_list
   * @action wp_ajax_nopriv_f_refresh_subscribers_list
   *
   * @author MS
   * @return void
   */
  public function refreshSubscribersLists()
  {
    if (isset($_POST['nonce']) && wp_verify_nonce($_POST['nonce'], 'f_refresh_subscribers_list')) {
      $data = self::getOptionsForSubscribersList(true);
      freshmail('notifications')->addNotification(__('Lists have been refreshed.', 'freshmail'), 'success', true);
      return wp_send_json_success([$data]);
    }
    return wp_send_json_error('Operation is not allowed!');
  }

  /**
   * Ajax: Save Settings
   *
   * @action wp_ajax_f_add_new_subscribers_list
   *
   * @author MS
   * @return void
   */
  public function addWPSubscribersList()
  {
    if (isset($_POST['nonce']) && wp_verify_nonce($_POST['nonce'], 'f_add_new_subscribers_list')) {
      $list = freshmail('api')->subscribers_list->createList();
      $data = self::getOptionsForSubscribersList(true);

      if (! empty($list)) {
        freshmail('notifications')->addNotification(__('A new list was added.', 'freshmail'), 'success', true);
        return wp_send_json_success([$list]);
      } else {
        freshmail('notifications')->addNotification(__('Could not add new list due to API error - please check your API token or try again later.', 'freshmail'), 'error', true);
        return wp_send_json_error('API error');
      }
    }
    return wp_send_json_error('Operation is not allowed!');
  }

  public function getOptionsForNewSubscriberStatus(): array
  {
    $return = [
      [
        'label' => __('Awaiting activation', 'freshmail'),
        'value' => 2,
      ],
      [
        'label' => __('Active', 'freshmail'),
        'value' => 1,
      ],
    ];
    return $return;
  }
}
