<?php

/**
 * Api: Api
 * Main API class
 *
 * @author MS
 */

namespace Freshmail\Core\Api;

use Freshmail\Core\Api\Submodules\EndpointsManager;
use Freshmail\Core\Api\Submodules\SubscribersListManager;

class Api
{
  /**
   * @var EndpointsManager
   * Contains all avaiable endpoints integrations
   */
  protected $endpoints = null;

  /**
   * @var SubscribersListManager
   */
  public $subscribers_list = null;

  public function __construct()
  {
    $this->init();
  }

  private function init(string $token = '')
  {
    if (function_exists('wp_doing_ajax') && ! wp_doing_ajax()) {
      $options_to_check = ['f_bearer_token', 'f_subscribers_list_hash'];
      $fail = false;
      foreach ($options_to_check as $option_name) {
        if (empty(get_option($option_name, ''))) {
          if ('f_subscribers_list_hash' === $option_name) {
            continue;
          }
          //cant access notifications now
          add_action(
              'init',
              function () use ($option_name) {
                $message = '';
                switch ($option_name) {
                  //MS: moving switch here to handle translations correctly
                  case 'f_bearer_token':
                        $message = __('FreshMail: Bearer token is missing - please paste it in the plugin options otherwise integrations wont work.', 'freshmail');
                        break;
                  case 'f_subscribers_list_hash':
                        // $message = __('FreshMail: Mailing list key is missing - please paste it in the plugin options otherwise integrations wont work.', 'freshmail');
                        break;
                  default:
                        $message = __('FreshMail: Plugin setup is missing some required values - please check the plugin options.', 'freshmail');
                        break;
                }
                freshmail('notifications')->addNotification($message, 'error', false, '', $option_name);
              }
          );
          $fail = true;
        }
      }

      if ($fail) {
        //do not continue if required options are missing
        return;
      }
    }

    $this->endpoints = new EndpointsManager();
    $this->subscribers_list = new SubscribersListManager($this->endpoints);
    // if (empty(get_option('f_subscribers_list_hash', ''))) {
    //   $this->subscribers_list->createList();
    // }
  }

  /**
   * Verifies the new token
   *
   * @action update_option_f_bearer_token
   * @param mixed  $old_value The old option value.
   * @param mixed  $value     The new option value.
   * @param string $option    Option name.
   * @return void
   * @author MS
   */
  public function checkBearerOnSave($old_value, $value, $option)
  {
    try {
      $this->init();
      $vendor = "WordPress";
      $version = get_bloginfo('version');
      if (freshmail('integrations')->woocommerce->isWCActive()) {
        $vendor = "WooCommerce";
        global $woocommerce;
        $version = $woocommerce->version;
      }
      $result = $this->endpoints->integrations->integrations($vendor, $version);
      add_settings_error('f_bearer_token', 'options_saved', __('Token was saved and verified.', 'freshmail'), 'success');
    } catch (\Throwable $th) {
      switch ($th->getCode()) {
        case 401:
              add_settings_error('f_bearer_token', 'options_saved', __('The token was saved but couldn\'t be verified - please check your token.', 'freshmail'), 'error');
              break;
        default:
              add_settings_error('f_bearer_token', 'options_saved', __('The token was saved but couldn\'t be verified due to API connection problem.', 'freshmail'), 'error');
              error_log('FRESHMAIL ERROR: API connection problem. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
              break;
      }
    }
  }

  /**
   * Removes notifications
   *
   * @action update_option
   * @param mixed  $old_value The old option value.
   * @param mixed  $value     The new option value.
   * @param string $option    Option name.
   * @return void
   * @author MS
   */
  public function removeNofitications($option, $old_value, $value)
  {
    if (0 === strpos($option, 'f_')) {
      freshmail('notifications')->removeNotification($option);
    }
  }

  /**
   * Checks if newle choosen list has all needed fields
   *
   * @filter pre_update_option_f_subscribers_list_hash
   * @param mixed  $old_value The old option value.
   * @param mixed  $value     The new option value.
   * @param string $option    Option name.
   * @return string
   * @author MS
   */
  public function checkListOnSave($value, $old_value, $option)
  {
    if (empty($value)) {
      return;
    }

    try {
      $this->subscribers_list->maybeAddNeededCustomFields($value);
      // freshmail('notifications')->addNotification(__('Token was saved and verified.', 'freshmail'), 'success', false);
    } catch (\Throwable $th) {
      switch ($th->getCode()) {
        case 401:
              add_settings_error('f_bearer_token', 'options_saved', __('The list was saved but couldn\'t be verified.', 'freshmail'), 'error');
              break;
        default:
              add_settings_error('f_bearer_token', 'options_saved', __('The list was saved but couldn\'t be verified due to API connection problem.', 'freshmail'), 'error');
              error_log('FRESHMAIL ERROR: API connection problem. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
              break;
      }
    }
    return $value;
  }

  public function isConnectionOk(): bool
  {
    try {
      $response = $this->endpoints->ping->ping();
      if (! empty($response['data']) && 'pong' === $response['data']) {
        return true;
      }
    } catch (\Throwable $th) {
      // do nothing
    }
    return false;
  }
}
