<?php
/**
 * Api: Subscribers list manager
 *
 * @author MS
 */
namespace Freshmail\Core\Api\Submodules;

use Freshmail\Core\Api\Endpoints\SubscribersList;

class SubscribersListManager
{
  /**
   * @var EndpointsManager|null
   * Contains all avaiable endpoints integrations
   */
  protected $endpoints = null;

  /**
   * @var array
   */
  private $custom_fields = [
    [
      'name' => 'imie',
      'tag' => 'imie',
      'type' => 0,
    ],
    [
      'name' => 'wp_nazwisko',
      'tag' => 'wp_nazwisko',
      'type' => 0,
    ],
    [
      'name' => 'wp_enroll_type',
      'tag' => 'wp_enroll_type',
      'type' => 0,
    ],
  ];

  public function __construct(EndpointsManager &$endpoints = null)
  {
    $this->endpoints = $endpoints;
  }

  /**
   * Creates new subscription list and saves the hash to DB to use it in other requests
   *
   * @param string $name created list name
   * @param string $description
   * @param array|null $custom_fields
   * @return null|array array with name and hash of created list, orherwise false
   * @author MS
   */
  public function createList(string $name = '', string $description = '', $custom_fields = null)
  {
    if (empty($name)) {
      $name = $this->getDefaultSubscribersListName();
    }

    if (empty($description)) {
      $description =  _x('Subscribers list for', 'Subscribers list description', 'freshmail') . ' ' . get_home_url();
    }

    if (null === $custom_fields) {
      $custom_fields = $this->custom_fields;
    }

    try {
      $result = $this->endpoints->subscribers_list->create($name, $description, $custom_fields);
      if (! empty($result['hash'])) {
        update_option('f_subscribers_list_hash', $result['hash']);
        return ['name' => $name, 'subscriberListHash' => $result['hash'], 'hash' => $result['hash']];
      }
    } catch (\Throwable $th) {
      freshmail('notifications')->addNotification(__('Could not create a new mailing list. Please try again.', 'freshmail'), 'error', false);
    }
    return null;
  }

  /**
   * Adds needed custom fields to the list
   *
   * @param string $hash subscribers list hash
   * @param array $fields field names to be added see $this->custom_fields array for names, if null all custom fields will be added
   * @return bool true if succeded otherwise false
   * @author MS
   */
  public function addNeededCustomFields(string $hash, $fieldsToAdd = null): bool
  {
    if (empty($hash)) {
      return false;
    }
    try {
      $fields = [];
      if (null === $fieldsToAdd) {
        $fields = $this->custom_fields;
      } else {
        foreach ($this->custom_fields as $field) {
          if (in_array($field['name'], $fieldsToAdd)) {
            $fields[] = $field;
          }
        }
      }

      $success = true;
      foreach ($fields as $field) {
        $field = wp_parse_args($field, ['name' => '', 'tag' => '', 'type' => 0]);
        $result = $this->endpoints->subscribers_list->addField($hash, $field['name'], $field['tag'], $field['type']);
        $success &= ! empty($result['status']) && 'ok' === $result['status'];
      }
      return $success;
    } catch (\Throwable $th) {
      // freshmail('notifications')->addNotification(__('Could not add new fields to the mailing list. Please try again.', 'freshmail'), 'error', false);
    }
    return false;
  }

  /**
   * Checks if needed custom fields are added to given subscribers list
   *
   * @param string $hash subscribers list to check
   * @return array|null array of missing field names, if array is empty then all fields are present, null - other error
   * @author MS
   */
  public function checkNeededCustomFields(string $hash)
  {
    if (empty($hash)) {
      return null;
    }
    try {
      $fields = $this->endpoints->subscribers_list->getFields($hash);
      if (array_key_exists('fields', $fields)) {
        $missing = [];
        $fields_to_check = array_column($fields['fields'], 'name');
        foreach ($this->custom_fields as $field) {
          if (! in_array($field['name'], $fields_to_check)) {
            $missing[] = $field['name'];
          }
        }
        return $missing;
      }
    } catch (\Throwable $th) {
      //throw $th;
    }
    return null;
  }

  /**
   * Checks and adds needed custom fields if any were missing
   *
   * @param string $hash list to check
   * @return void
   * @author MS
   */
  public function maybeAddNeededCustomFields(string $hash)
  {
    if (empty($hash)) {
      return;
    }

    $missing = $this->checkNeededCustomFields($hash);
    if (! empty($missing)) {
      $this->addNeededCustomFields($hash, $missing);
    }
  }

  /**
   * Returns avaiable subscribers lists
   *
   * @return array can be empty
   * @author MS
   */
  public function getAvailableLists(): array
  {
    try {
      $lists = $this->endpoints->subscribers_list->lists();
      if (! empty($lists['lists'])) {
        return $lists['lists'];
      }
    } catch (\Throwable $th) {
      //throw $th;
    }
    return [];
  }

  /**
   * Adds a new subscriber to given subscribers list
   *
   * @param string $email
   * @param string $hash list ID where new subscriber should be added
   * @param int $state status of subscriber, if 0 then populated from settings at later stage
   * @param int $confirm If subscriber is to be send confirmation mail or not. Possible values 0 or 1, if -1 then it will be automatically set based on $state.
   * @param array|null $custom_fields
   * @return bool true if subscriber was created, otherwise false
   * @author MS
   */
  public function addSubscriberToList(string $email, $custom_fields = null, string $hash = '', int $state = 0, int $confirm = -1): bool
  {
    if (! filter_var($email, FILTER_VALIDATE_EMAIL)) {
      return false;
    }

    try {
      $result = $this->endpoints->subscriber->add($email, $hash, $custom_fields, $state, $confirm);
      if (! empty($result['errors'])) {
        return false;
      }
    } catch (\Throwable $th) {
      error_log('FRESHMAIL ERROR: Could not add user ' . substr_replace($email, '(...)', 3, strrpos($email, '.') - 5) . ' to list ' . $hash . '. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
      return false;
    }

    return true;
  }

  /**
   * Updates a subscriber at given subscribers list.
   * Will add a new subscriber if the subscriber wasnt added before.
   *
   * @param string $email
   * @param string $hash list ID where new subscriber should be added
   * @param int $state status of subscriber, if 0 then populated from settings at later stage
   * @param int $confirm If subscriber is to be send confirmation mail or not. Possible values 0 or 1, if -1 then it will be automatically set based on $state.
   * @param array|null $custom_fields
   * @param bool $add_to_list Wether a new user should be added in case if it doesn't already exists or not
   * @return bool true if subscriber was created, otherwise false
   * @author MS
   */
  public function updateSubscriberAtList(string $email, $custom_fields = null, string $hash = '', int $state = 0, int $confirm = -1, bool $add_to_list = true): bool
  {
    if (! filter_var($email, FILTER_VALIDATE_EMAIL)) {
      return false;
    }

    $custom_fields = $this->cleanEmptyArrayKeys($custom_fields);

    try {
      $result = $this->endpoints->subscriber->edit($email, $hash, $custom_fields, $state, $confirm);
      if (! empty($result['errors'])) {
        if ($add_to_list) {
          foreach ($result['errors'] as $error) {
            if (! empty($error['code']) && 1331 === $error['code']) {
              return $this->addSubscriberToList($email, $custom_fields, $hash, $state, $confirm);
            }
          }
        } else {
          return false;
        }
      }
    } catch (\Throwable $th) {
      error_log('FRESHMAIL ERROR: Could not update user ' . substr_replace($email, '(...)', 3, strrpos($email, '.') - 5) . ' at list ' . $hash . '. Message: [' . $th->getMessage() . '] Code: ' . $th->getCode());
      return false;
    }

    return true;
  }

  /**
   * Removes empty keys from array
   *
   * @param array $array
   * @return array
   * @author MS
   */
  private function cleanEmptyArrayKeys(array $array): array
  {
    if (! empty($array)) {
      foreach ($array as $index => $element) {
        if (empty($element)) {
          unset($array[$index]);
        }
      }
    }
    return $array;
  }

  /**
   * Returns default subscribers list name
   *
   * @return string
   * @author MS
   */
  public function getDefaultSubscribersListName(): string
  {
    return _x('WordPress List', 'Subscribers list name', 'freshmail') . ' - ' . get_bloginfo('name');
  }
}
