<?php

/**
 * Integrations: WooCommerce
 * Main WooCommerce class
 *
 * @author MS
 */

namespace Freshmail\Core\Integrations\WooCommerce;

use Freshmail\Core\Integrations\Integration;

class WooCommerce extends Integration
{
  /**
   * @var WooCommerce
   */
  private $integration = null;

  public function __construct()
  {
    if ($this->isEnabled()) {
      $this->integration = createClass('Freshmail\Core\Integrations\WooCommerce\WooCommerceIntegration\WooCommerce');
    }

    add_filter('pre_update_option_' . $this->getOptionName('enabled'), [$this, 'watchEnableChanges'], 10, 3);
    add_filter('pre_update_option_' . $this->getOptionName('consent'), [$this, 'watchConsentContent'], 10, 3);
  }

  /**
   * Init
   *
   * @action init
   *
   * @author author
   * @return void
   */
  public function init()
  {
    $this->name = _x('WooCommerce integration', 'integration name', 'freshmail');
  }

  /**
   * Returns integration sections for settings screen.
   * See AdminMenu and IntegrationsPage core module for more info
   *
   * @return array
   */
  public function getSettingsSections(): array
  {
    return [
      [
        'id' => $this::OPTION_PREFIX . 'wc_main_settings_section',
        'title' => $this->name, //__('SMTP integration', 'freshmail'),
        'page' => 'freshmail-integrations',
      ],
    ];
  }

  /**
   * Returns integration fields for settings screen.
   * See AdminMenu and IntegrationsPage core module for more info
   *
   * @return array
   */
  public function getSettingsFields(): array
  {
    return [
      [
        'label' => __('Newsletter signup checkbox', 'freshmail'),
        'type' => 'text',
        'default_value' => __('Subscribe newsletter to stay updated with new products and offers!', 'freshmail'),
        'name' => $this->getOptionName('consent'),
        'id' => $this->getOptionName('consent'),
        'css_class' => 'wc__setting-field wc__setting-field--wide',
        'placeholder' => __('Type in consent text here', 'freshmail'),
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'wc_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row__wc_consent hide-if-section-disabled'],
        'description' => __('This text will be displayed to users when creating account or when on checkout if they haven\'t already agreed. It is also used in the WooCommerce my account settings.', 'freshmail'),
        // 'readonly' => 'readonly',
      ],
      [
        'label' => __('Enabled', 'freshmail'),
        'type' => 'checkbox',
        'name' => $this->getOptionName('enabled'),
        'id' => $this->getOptionName('enabled'),
        'css_class' => 'wc__setting-field',
        'placeholder' => '',
        'page' => 'freshmail-integrations',
        'section' => $this::OPTION_PREFIX . 'wc_main_settings_section',
        'args' => ['class' => 'settings-field-row settings-field-row__wc_enabled'],
      ],
    ];
  }

  public function getOptionName(string $option): string
  {
    return $this::OPTION_PREFIX . 'wc_' . $option;
  }

  /**
   * Checks if integration can be enabled.
   *
   * @return bool true if integration can be enabled otherwise false
   * @author MS
   */
  public function canBeEnabled(): bool
  {
    return $this->isWCActive();
  }

  /**
   * Watches changes of enabled setting.
   * If integration is being enabled checks the needed requirements and enables it.
   * If check fails the new value is not saved.
   * Runs on pre_update_option_{enabled} - see constructor
   *
   * @param mixed  $value     The new, unserialized option value.
   * @param mixed  $old_value The old option value.
   * @param string $option    Option name.
   * @return mixed
   * @author MS
   */
  public function watchEnableChanges($value, $old_value, string $option)
  {
    if (empty($value)) {
      //check only when enabling
      return $value;
    }

    if ($this->canBeEnabled()) {
      return $value;
    } else {
      if (! $this->isWCActive()) {
        $message = __('Can\'t enable %s - WooCommerce plugin is not activated.', 'freshmail');
      } else {
        $message = __('Can\'t enable %s - some of reqired data is missing.', 'freshmail');
      }
      $message = sprintf($message, $this->getName());
      freshmail('notifications')->addNotification($message, 'warning', false);
    }
    return $old_value;
  }

  /**
   * Checks if integration can still work after updating some of its options
   *
   * @action updated_option
   * @param mixed  $old_value The old option value.
   * @param mixed  $value     The new option value.
   * @param string $option    Option name.
   * @return void
   * @author MS
   */
  public function checkIfCanStayEnabled($option, $old_value, $value)
  {
    if (0 === strpos($option, $this::OPTION_PREFIX . 'wc_')) {
      if (!$this->canBeEnabled()) {
        $this->disable();
      }
    }
  }

  /**
   * Checks if WooCommerce is activated
   *
   * @return bool true if activated, otherwise false
   * @author MS
   */
  public function isWCActive(): bool
  {
    $activated_plugins = wp_get_active_and_valid_plugins();
    if (function_exists('wp_get_active_network_plugins')) {
        $activated_plugins = array_merge(wp_get_active_network_plugins(), wp_get_active_and_valid_plugins());
    }

    foreach ($activated_plugins as $plugin_path) {
      if (false !== strpos($plugin_path, 'plugins/woocommerce/woocommerce.php')) {
        return true;
      }
    }
    return false;
  }

  /**
   * Watches if WooCommerce was deactivated.
   * If yes then deactivates integration.
   *
   * @action deactivated_plugin
   * @param string $plugin
   * @param bool $network_wide
   * @return void
   * @author MS
   */
  public function watchWCDeactivation($plugin, $network_wide)
  {
    if (false !== strpos($plugin, 'woocommerce/woocommerce.php')) {
      $this->disable();
    }
  }

  /**
   * Tries to enable integration if possible
   *
   * @action activated_plugin
   * @return bool true if activated, oterwise false
   * @author MS
   */
  public function maybeEnableIntegration($plugin, $network_activation): bool
  {
    if (false !== strpos($plugin, 'freshmail/freshmail.php') && ! $this->wasEnabledBefore() && $this->isWCActive()) {
      return $this->enableFirstTime();
    }
    return false;
  }

  /**
   * Tells if integration was enabled before or if next activation will be first one.
   *
   * @return bool false if next activation would be the first one, otherwise true
   * @author MS
   */
  public function wasEnabledBefore(): bool
  {
    if (empty(get_option($this->getOptionName('first_enable_done'), null))) {
      return false;
    }
    return true;
  }

  /**
   * Performs first activation of integration (enables it first time after plugin instalation).
   *
   * @return bool
   * @author MS
   */
  public function enableFirstTime(): bool
  {
    update_option($this->getOptionName('first_enable_done'), true);
    return $this->enable();
  }

  /**
   * Returns consent content that was set by user.
   *
   * @return string
   * @author MS
   */
  public function getConsentContent(): string
  {
    $default = '';
    $option_name = $this->getOptionName('consent');
    $fields = $this->getSettingsFields();

    foreach ($fields as $field) {
      if ($field['name'] === $option_name) {
        $default = $field['default_value'];
        break;
      }
    }

    $return = get_option($option_name, $default);
    return empty($return) ? $default : $return;
  }

  /**
   * Watches changes of consent content setting - we need some text always there.
   * Runs on pre_update_option_{consent} - see constructor
   *
   * @param mixed  $value     The new, unserialized option value.
   * @param mixed  $old_value The old option value.
   * @param string $option    Option name.
   * @return mixed
   * @author MS
   */
  public function watchConsentContent($value, $old_value, string $option)
  {
    if (empty($value)) {
      $option_name = $this->getOptionName('consent');
      $fields = $this->getSettingsFields();

      foreach ($fields as $field) {
        if ($field['name'] === $option_name) {
          return $field['default_value'];
        }
      }
    }

    return $value;
  }
}
