<?php

/**
 * Integrations: WooCommerce
 * Main WooCommerce class
 *
 * @author MS
 */

namespace Freshmail\Core\Integrations\WooCommerce\WooCommerceIntegration;

use Freshmail\Core\Integrations\Integration;
use WC_Order;

class WooCommerce
{
  /**
   * Adds FreshMail extra fields on native WC registration form (login and checkout).
   *
   * @action woocommerce_register_form
   * @disabled_action woocommerce_after_checkout_registration_form
   * @return type description
   * @author MS
   */
  public function addRegistrationExtraFields()
  {
    // include FRERSHMAIL_RESOURCES_PATH . 'views/woocommerce/registration-fields.php';
    woocommerce_form_field(
        'f_api_use_consent',
        [
          'type'         => 'checkbox',
          'label'        => freshmail('integrations')->woocommerce->getConsentContent(),
          'required'     => false,
          'autocomplete' => false,
          'id'           => 'f_api_use_consent',
          'return'       => false,
          'class'        => ['f-register-field', 'f-register-field--checkbox'],
        ],
        'yes' === get_user_meta(get_current_user_id(), 'f_api_use_consent', true)
    );
  }

  /**
   * Saves values of extra fields and adds subscriber to list if user agreed.
   *
   * @action woocommerce_created_customer 99
   * @action woocommerce_save_account_details
   * @return void
   * @author MS
   */
  public function saveRegistrationExtraFields($customer_id)
  {
    if (isset($_POST['f_api_use_consent'])) {
      update_user_meta($customer_id, 'f_api_use_consent', 'on' === $_POST['f_api_use_consent'] || 1 == $_POST['f_api_use_consent'] ? 'yes' : 'no');
      if ('on' === $_POST['f_api_use_consent'] || 1 == $_POST['f_api_use_consent']) {
        $user = wp_get_current_user();
        $email = empty($_POST['email']) ? $user->user_email : $_POST['email'];
        $custom_fields = [
          'imie' => $user->first_name,
          'wp_nazwisko' => $user->last_name,
        ];
        if (! empty($_POST['register'])) {
          $custom_fields['wp_enroll_type'] = 'utworzenie konta';
        }
        freshmail('api')->subscribers_list->updateSubscriberAtList($email, $custom_fields);
      }
    } else {
      //save denial of sending data
      update_user_meta($customer_id, 'f_api_use_consent', 'no');
    }
  }

  /**
   * Adds extra fields to WC account edit screen
   *
   * @action woocommerce_edit_account_form
   * @return void
   * @author MS
   */
  public function addExtraFieldsToWcAccountEdit()
  {
    woocommerce_form_field(
        'f_api_use_consent',
        [
          'type'         => 'checkbox',
          'label'        => freshmail('integrations')->woocommerce->getConsentContent(),
          'required'     => false,
          'autocomplete' => false,
          'id'           => 'f_api_use_consent',
          'return'       => false,
          'class'        => ['f-account-field', 'f-account-field--checkbox'],
        ],
        'yes' === get_user_meta(get_current_user_id(), 'f_api_use_consent', true)
    );
  }

  /**
   * Adds checkout checboxes
   *
   * @action woocommerce_review_order_before_submit
   * @return void
   * @author MS
   */
  public function addCheckoutCheckboxes()
  {
    if ($this->shouldDisplayCheckoutCheckboxes()) {
      woocommerce_form_field('f_checkout_consent', array(
        'type'          => 'checkbox',
        'class'         => array('form-row mycheckbox'), // CSS Class
        'label_class'   => array('woocommerce-form__label woocommerce-form__label-for-checkbox checkbox f-checkout-field f-checkout-field--checkbox'),
        'input_class'   => array('woocommerce-form__input woocommerce-form__input-checkbox input-checkbox'),
        'required'      => false, // Mandatory or Optional
        'label'         => freshmail('integrations')->woocommerce->getConsentContent(),
      ));
    }
  }

  /**
   * Tells if checkout checkboxes should be displayed.
   * They should be hidden if user already agreed to sending data.
   *
   * @return bool true if checkboxes should be displayed, false otherwise
   * @author MS
   */
  public function shouldDisplayCheckoutCheckboxes(): bool
  {
    if (is_user_logged_in() && $this->userHasAgreedToSendingData(get_current_user_id())) {
      return false;
    }
    return true;
  }

  /**
   * Tells if user has agreed to sending data to FreshMail
   *
   * @param int $user_id
   * @return bool true if user has agreed, false otherwise
   * @author MS
   */
  public function userHasAgreedToSendingData($user_id): bool
  {
    if ('yes' === get_user_meta($user_id, 'f_api_use_consent', true)) {
      return true;
    }
    return false;
  }

  /**
   * Fires FreshMail synchronization if user agreed
   *
   * @action woocommerce_new_order
   * @param int $order_id
   * @param WC_Order $order
   * @return void
   * @author MS
   */
  public function handleNewOrder(int $order_id, WC_Order $order)
  {
    if (isset($_POST['f_checkout_consent'])) {
      update_post_meta($order_id, 'f_checkout_consent', 'yes');
      $custom_fields = [
        'imie' => $order->get_billing_first_name(),
        'wp_nazwisko' => $order->get_billing_last_name(),
      ];

      if (! $this->userHasAgreedToSendingData(get_current_user_id())) {
        update_user_meta(get_current_user_id(), 'f_api_use_consent', 'yes');
        $custom_fields['wp_enroll_type'] = 'zakup';
      }

      $result = freshmail('api')->subscribers_list->updateSubscriberAtList($order->get_billing_email(), $custom_fields);
    } elseif ('yes' === get_user_meta(get_current_user_id(), 'f_api_use_consent', true)) {
      update_post_meta($order_id, 'f_checkout_consent', 'agreed_before');
      $custom_fields = [
        'imie' => $order->get_billing_first_name(),
        'wp_nazwisko' => $order->get_billing_last_name(),
      ];

      $result = freshmail('api')->subscribers_list->updateSubscriberAtList($order->get_billing_email(), $custom_fields);
    }
  }
}
